@@ -1,16 +1,16 @@
-import os
-import re
 import shutil
 import typing
 
 import numpy as np
+from abageotests import AbaqusTangentialContactType
 from matplotlib import pyplot as plt
 
 from ..ababase.AbaqusGeometry import AbaqusGeometry
 from ..ababase.AbaqusMesh import AbaqusMeshMethod
-from ..ababase.AbaqusModelBase import AbaqusModelBase
-from ..ababase.AbaqusStep import AbaqusCalculationMethod
+from ..ababase.AbaqusStep import AbaqusCalculationMethod, AbaqusBaseStepIncrementationType
 from ..ababase.AbaqusSubroutine import AbaqusSubroutineType
+from ..abageoabstract.AbaGeoTestBase import AbaGeoTestBase
+from ..ababase.abaqusConstants import *
 
 
 class AbaqusDirectShearTestGeometry(AbaqusGeometry):
@@ -27,7 +27,7 @@ class AbaqusDirectShearTestGeometry(AbaqusGeometry):
         self.length, self.width, self.height = length, width, height
 
 
-class AbaqusDirectShearTest(AbaqusModelBase):
+class AbaqusDirectShearTest(AbaGeoTestBase):
     """
     This is a python class to manage the information of the direct shear test, and to generate python script to build
     the abaqus model. You can use this class to
@@ -55,39 +55,41 @@ class AbaqusDirectShearTest(AbaqusModelBase):
                  n_state_dependent_variables: int = 0, friction_properties: tuple = None):
 
         super().__init__()
-        self.setCalculationMethod(calculation_method)
+        self.CalculationMethod(calculation_method)
         self.SoilGeometry(*soil_geometry)
         self.SolidGeometry(*solid_geometry)
         self.SoilMaterial(*soil_material)
         self.SolidMaterial(*solid_material)
         self.SoilMesh(soil_mesh_method, soil_seeds)
         self.SolidMesh(solid_mesh_method, solid_seeds)
-        self.VerticalPressure(vertical_pressure)
-        self.PredefinedConfiningStress(*predefined_stress)
-        self.DefaultHistoryOutput(['U1', 'RF1'], 10)
+        self.Pressure(vertical_pressure)
+        self.PredefinedStress(*predefined_stress)
+        self.HistoryOutput(['U1', 'RF1'], 10)
 
         if calculation_method == AbaqusCalculationMethod.Standard:
-            self.SoilDisplacement(*displacement)
-            self.GeneralStaticStep('Initial-Stress', time_period=initial_stress_time_period,
-                                   description=initial_stress_description)
-            self.GeneralStaticStep('Shear', time_period=shear_time_period, description=shear_description)
+            self.Displacement(*displacement)
+            self.InitialStressGeneralStaticStep(time_period=initial_stress_time_period,
+                                                description=initial_stress_description)
+            self._GeneralStaticStep('Initial-Stress', time_period=initial_stress_time_period,
+                                    description=initial_stress_description)
+            self.ShearGeneralStaticStep(time_period=shear_time_period, description=shear_description)
             self.FrictionSubroutine(AbaqusSubroutineType.FRIC, subroutine_filepath=subroutine_filepath,
                                     n_state_dependent_variables=n_state_dependent_variables,
                                     friction_properties=friction_properties)
-            self.DefaultFieldOutput(['S', 'E', 'LE', 'U', 'RF', 'RT', 'RM', 'P',
-                                     'CSTRESS', 'CDISP', 'CFORCE', 'CNAREA', 'CSTATUS'])
+            self.FieldOutput(['S', 'E', 'LE', 'U', 'RF', 'RT', 'RM', 'P',
+                              'CSTRESS', 'CDISP', 'CFORCE', 'CNAREA', 'CSTATUS'])
         else:
-            self.SoilVelocity(*velocity)
-            self.ExplicitDynamicStep('Initial-Stress', time_period=initial_stress_time_period,
-                                     description=initial_stress_description)
-            self.ExplicitDynamicStep('Shear', time_period=shear_time_period, description=shear_description)
+            self.Velocity(*velocity)
+            self.InitialStressExplicitDynamicStep(time_period=initial_stress_time_period,
+                                                  description=initial_stress_description)
+            self.ShearExplicitDynamicStep(time_period=shear_time_period, description=shear_description)
             self.FrictionSubroutine(AbaqusSubroutineType.VFRIC, subroutine_filepath=subroutine_filepath,
                                     n_state_dependent_variables=n_state_dependent_variables,
                                     friction_properties=friction_properties)
-            self.DefaultFieldOutput(['S', 'E', 'LE', 'U', 'V', 'A', 'RF', 'P',
-                                     'CSTRESS', 'CFORCE', 'FSLIPR', 'FSLIP', 'PPRESS'])
+            self.FieldOutput(['S', 'E', 'LE', 'U', 'V', 'A', 'RF', 'P',
+                              'CSTRESS', 'CFORCE', 'FSLIPR', 'FSLIP', 'PPRESS'])
 
-    def Geometry(self, name: str, length: float = 1.0, width: float = 1.0, height: float = 1.0):
+    def _Geometry(self, name: str = 'Soil', length: float = 1.0, width: float = 1.0, height: float = 1.0):
         """Set geometry of the soil and solid
 
         Parameters
@@ -125,7 +127,7 @@ class AbaqusDirectShearTest(AbaqusModelBase):
         -------
         None
         """
-        self.Geometry('Soil', length, width, height)
+        self._Geometry('Soil', length, width, height)
 
     def SolidGeometry(self, length: float, width: float, height: float):
         """Set geometry of the solid, identical to Geometry('Solid', length, width, height)
@@ -143,7 +145,7 @@ class AbaqusDirectShearTest(AbaqusModelBase):
         -------
         None
         """
-        self.Geometry('Solid', length, width, height)
+        self._Geometry('Solid', length, width, height)
 
     def SoilMaterial(self, modulus: float, poisson_ratio: float, density: float = None):
         """Set soil properties, identical to Material('Soil', modulus, poisson_ratio, density)
@@ -160,7 +162,7 @@ class AbaqusDirectShearTest(AbaqusModelBase):
         -------
         None
         """
-        self.Material('Soil', modulus, poisson_ratio, density)
+        self._Material('Soil', modulus, poisson_ratio, density)
 
     def SolidMaterial(self, modulus: float, poisson_ratio: float, density: float = None):
         """Set solid properties, identical to Material('Solid', modulus, poisson_ratio, density)
@@ -177,189 +179,10 @@ class AbaqusDirectShearTest(AbaqusModelBase):
         -------
         None
         """
-        self.Material('Solid', modulus, poisson_ratio, density)
-
-    def VerticalPressure(self, amplitude: typing.Union[float, typing.Iterable]):
-        """Set vertical pressure, identical to Pressure('Vertical-Pressure', amplitude)
-
-        Parameters
-        ----------
-        amplitude: float
-            amplitude of the vertical pressure
-
-        Returns
-        -------
-        None
-        """
-        self.Pressure('Vertical-Pressure', amplitude)
+        self._Material('Solid', modulus, poisson_ratio, density)
 
     @typing.overload
-    def PredefinedConfiningStress(self, predefined_stress: typing.Iterable):
-        """Set predefined stress, identical to PredefinedStress('Predefined-Confining-Stress', predefined_stress)
-
-        Parameters
-        ----------
-        predefined_stress: typing.Iterable
-            predefined stress, an array with the dimension of 6.
-        Returns
-        -------
-        None
-        """
-        pass
-
-    @typing.overload
-    def PredefinedConfiningStress(self, sig11: float, sig22: float, sig33: float, sig12: float, sig13: float,
-                                  sig23: float):
-        """Set predefined stress, identical to
-        PredefinedStress('Predefined-Confining-Stress', sig11, sig22, sig33, sig12, sig13, sig23)
-
-        Parameters
-        ----------
-        sig11: float
-            component sig11 of the predefined stress
-        sig22: float
-            component sig22 of the predefined stress
-        sig33: float
-            component sig33 of the predefined stress
-        sig12: float
-            component sig12 of the predefined stress
-        sig13: float
-            component sig13 of the predefined stress
-        sig23: float
-            component sig23 of the predefined stress
-
-        Returns
-        -------
-        None
-        """
-        pass
-
-    def PredefinedConfiningStress(self, *args):
-        """Set predefined stress, identical to PredefinedStress('Predefined-Confining-Stress', *args)
-
-        Parameters
-        ----------
-        args: typing.Iterable or float, float, float, float, float, float
-            arguments of the predefined stress, either an array with the dimension of 6, or 6 components of the predefined
-            stress
-
-        Returns
-        -------
-        None
-        """
-        self.PredefinedStress('Predefined-Confining-Stress', *args)
-
-    @typing.overload
-    def SoilDisplacement(self, displacement: typing.Iterable):
-        """Set displacement of the shearing, identical to Displacement('Displacement', displacement)
-
-        Parameters
-        ----------
-        displacement: typing.Iterable
-            shearing displacement, an array with the dimension of 6
-        Returns
-        -------
-        None
-        """
-        pass
-
-    @typing.overload
-    def SoilDisplacement(self, u1: float = 0, u2: float = 0, u3: float = 0, ur1: float = 0, ur2: float = 0,
-                         ur3: float = 0):
-        """Set displacement of the shearing, identical to Displacement('Displacement', u1, u2, u3, ur1, ur2, ur3)
-
-        Parameters
-        ----------
-        u1: float
-            component u1 of the displacement
-        u2: float
-            component u2 of the displacement
-        u3: float
-            component u3 of the displacement
-        ur1: float
-            component ur1 of the displacement
-        ur2: float
-            component ur2 of the displacement
-        ur3: float
-            component ur3 of the displacement
-
-        Returns
-        -------
-        None
-        """
-
-        pass
-
-    def SoilDisplacement(self, *args):
-        """Set displacement of the shearing, identical to Displacement('Displacement', *args)
-
-        Parameters
-        ----------
-        args: typing.Iterable or float, float, float, float, float, float
-            arguments of the displacement, either an array with the dimension of 6, or 6 components of the displacement
-
-        Returns
-        -------
-        None
-        """
-        self.Displacement('Displacement', *args)
-
-    @typing.overload
-    def SoilVelocity(self, velocity: typing.Iterable):
-        """Set velocity of the shearing, identical to Velocity('Velocity', velocity)
-
-        Parameters
-        ----------
-        velocity: typing.Iterable
-            shearing velocity, an array with the dimension of 6
-        Returns
-        -------
-        None
-        """
-        pass
-
-    @typing.overload
-    def SoilVelocity(self, v1: float = 0, v2: float = 0, v3: float = 0, vr1: float = 0, vr2: float = 0,
-                     vr3: float = 0):
-        """Set velocity of the shearing, identical to Velocity('Velocity', v1, v2, v3, vr1, vr2, vr3)
-
-        Parameters
-        ----------
-        v1: float
-            component v1 of the velocity
-        v2: float
-            component v2 of the velocity
-        v3: float
-            component v3 of the velocity
-        vr1: float
-            component vr1 of the velocity
-        vr2: float
-            component vr2 of the velocity
-        vr3: float
-            component vr3 of the velocity
-
-        Returns
-        -------
-        None
-        """
-        pass
-
-    def SoilVelocity(self, *args):
-        """Set velocity of the shearing, identical to Velocity('Velocity', *args)
-
-        Parameters
-        ----------
-        args: typing.Iterable or float, float, float, float, float, float
-            arguments of the displacement, either an array with the dimension of 6, or 6 components of the velocity
-
-        Returns
-        -------
-        None
-        """
-        self.Velocity('Velocity', *args)
-
-    @typing.overload
-    def SoilMesh(self, method: AbaqusMeshMethod.ByNumber, seeds_number: int):
+    def SoilMesh(self, method=AbaqusMeshMethod.ByNumber, seeds_number: int = 1):
         """Set mesh parameter of the soil by number of seeds, identical to Mesh('Soil', method, seeds_number)
 
         Parameters
@@ -375,7 +198,7 @@ class AbaqusDirectShearTest(AbaqusModelBase):
         pass
 
     @typing.overload
-    def SoilMesh(self, method: AbaqusMeshMethod.BySize, seeds_size: float):
+    def SoilMesh(self, method=AbaqusMeshMethod.BySize, seeds_size: float = 0.01):
         """Set mesh parameter of the soil by size of seeds, identical to Mesh('Soil', method, seeds_size)
 
         Parameters
@@ -390,7 +213,7 @@ class AbaqusDirectShearTest(AbaqusModelBase):
         """
         pass
 
-    def SoilMesh(self, method: AbaqusMeshMethod, seeds: typing.Union[int, float]):
+    def SoilMesh(self, method=AbaqusMeshMethod.ByNumber, seeds: typing.Union[int, float] = 1):
         """Set mesh parameter of the soil by number/size of seeds, identical to Mesh('Soil', method, seeds)
 
         Parameters
@@ -403,10 +226,10 @@ class AbaqusDirectShearTest(AbaqusModelBase):
         -------
         None
         """
-        self.Mesh('Soil', method, seeds)
+        self._GeneralMesh('Soil', method, seeds)
 
     @typing.overload
-    def SolidMesh(self, method: AbaqusMeshMethod.ByNumber, seeds_number: int):
+    def SolidMesh(self, method=AbaqusMeshMethod.ByNumber, seeds_number: int = 1):
         """Set mesh parameter of the solid by number of seeds, identical to Mesh('Solid', method, seeds_number)
 
         Parameters
@@ -422,7 +245,7 @@ class AbaqusDirectShearTest(AbaqusModelBase):
         pass
 
     @typing.overload
-    def SolidMesh(self, method: AbaqusMeshMethod.BySize, seeds_size: float):
+    def SolidMesh(self, method=AbaqusMeshMethod.BySize, seeds_size: float = 0.01):
         """Set mesh parameter of the solid by size of seeds, identical to Mesh('Solid', method, seeds_size)
 
         Parameters
@@ -437,7 +260,7 @@ class AbaqusDirectShearTest(AbaqusModelBase):
         """
         pass
 
-    def SolidMesh(self, method: AbaqusMeshMethod, seeds: typing.Union[int, float]):
+    def SolidMesh(self, method=AbaqusMeshMethod.ByNumber, seeds: typing.Union[int, float] = 1):
         """Set mesh parameter of the solid by number/size of seeds, identical to Mesh('Solid', method, seeds)
 
         Parameters
@@ -450,123 +273,94 @@ class AbaqusDirectShearTest(AbaqusModelBase):
         -------
         None
         """
-        self.Mesh('Solid', method, seeds)
+        self._GeneralMesh('Solid', method, seeds)
 
     @typing.overload
-    def FrictionSubroutine(self, subroutine_type=AbaqusSubroutineType.FRIC, subroutine_filepath: str = '',
-                           n_state_dependent_variables: int = 0, cohesion: float = 0.0, G0: float = 250.0,
-                           R: float = 5.19, e_cref: float = 0.89, Lambda: float = 0.147, xi: float = 0.424,
-                           phi: float = 31.2, dd: float = 7.57, n_p: float = 2.06, n_d: float = 0.46,
-                           e0: float = 0.5771, thickness: float = 5 * 0.23 * 1e-3):
-        """Set properties of the exponential interface model (FRIC/VFRIC) subroutine,
-
-        Common properties
-
-        - Dense: [0.0,  250.0, 5.19, 0.89, 0.147, 0.424, 31.2, 7.57, 2.06,  0.46, 0.5771, 0.00115]
-        - Loose: [0.0,  290.0, 2.69, 1.20, 0.750, 0.161, 29.5, 14.2, 0.10,  0.10, 0.7700, 0.00115]
-        - C:     [0.0, 2000.0, 6.96, 0.90, 0.413, 1.000, 36.4, 0.10, 5.70, 14.01, 0.5771, 0.00500]
-
-        Parameters
-        ----------
-        subroutine_type: AbaqusSubroutineType
-            subroutine type, either AbaqusSubroutineType.FRIC or AbaqusSubroutineType.VFRIC
-        subroutine_filepath: str
-            filepath of the subroutine
-        n_state_dependent_variables: int
-            number of dependent variables
-        cohesion: float
-            cohesion of the soil
-        G0: float
-            initial shear modulus
-        R: float
-            a material constant
-        e_cref: float
-            reference void ratio
-        Lambda: float
-            parameter controlling the shape of CSL line
-        xi: float
-            parameter controlling the nonlinearity of CSL line
-        phi: float
-            friction angle
-        dd: float
-            a dilatancy parameter
-        n_p: float
-            a parameter in the critical state concept
-        n_d: float
-            a parameter in the critical state concept
-        e0: float
-            initial void ratio
-        thickness: float
-            thickness of the soil-structure interface
-
-        Returns
-        -------
-        None
-        """
+    def InitialStressGeneralStaticStep(self, time_period: float = 1.0, description: str = '',
+                                       incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
+                                       maximal_increments: int = 100, initial_increment_size: float = 1.0,
+                                       minimal_increment_size: float = 1e-5, maximal_increment_size: float = 1.0):
         pass
 
     @typing.overload
-    def FrictionSubroutine(self, subroutine_type=AbaqusSubroutineType.FRIC, subroutine_filepath: str = '',
-                           n_state_dependent_variables: int = 0, friction_properties: typing.Iterable = None):
-        """Set properties of the FRIC/VFRIC subroutine
-
-        Parameters
-        ----------
-        subroutine_type: AbaqusSubroutineType
-            subroutine type, either AbaqusSubroutineType.FRIC or AbaqusSubroutineType.VFRIC
-        subroutine_filepath: str
-            filepath of the subroutine
-        n_state_dependent_variables: int
-            number of dependent variables
-        friction_properties: typing.Iterable
-            friction properties in the FRIC/VFRIC subroutine
-        Returns
-        -------
-        None
-        """
+    def InitialStressGeneralStaticStep(self, time_period: float = 1.0, description: str = '',
+                                       incrementation_type=AbaqusBaseStepIncrementationType.Fixed,
+                                       maximal_increments: int = 100, fixed_increment_size: float = 0.1):
         pass
 
-    def FrictionSubroutine(self, subroutine_type=AbaqusSubroutineType.FRIC, subroutine_filepath: str = '',
-                           n_state_dependent_variables: int = 0, *args, **kwargs):
-        """Set properties of the FRIC/VFRIC subroutine
+    def InitialStressGeneralStaticStep(self, time_period: float = 1.0, description: str = '',
+                                       incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
+                                       *args, **kwargs):
+        self._GeneralStaticStep('Initial-Stress', time_period, description, incrementation_type, *args, **kwargs)
 
-        Parameters
-        ----------
-        subroutine_type: AbaqusSubroutineType
-            subroutine type, either AbaqusSubroutineType.FRIC or AbaqusSubroutineType.VFRIC
-        subroutine_filepath: str
-            filepath of the subroutine
-        n_state_dependent_variables: int
-            number of dependent variables
-        args, kwargs
-            friction properties in the FRIC/VFRIC subroutine, if you use non-key-value way to call this function, please
-            provide in correct order, if you use the key-value way to call this function, please provide all the
-            properties, if you call this function in a mixed way, please provide the properties in correct order in
-            non-key-value and it must be the first several properties, please provide the rest properties in key-value
-            way. Whatever in what way you use, all parameters must be specified, otherwise an error would occur.
-        Returns
-        -------
-        None
-        """
-        if len(args) + len(kwargs) == 1:
-            friction_properties = args[0] if len(args) == 1 else kwargs['friction_properties']
-        else:
-            keys = ['cohesion', 'G0', 'R', 'e_cref', 'Lambda', 'xi', 'phi', 'dd', 'n_p', 'n_d', 'e0', 'thickness']
-            for arg, key in zip(args, keys[:len(args)]):
-                kwargs[key] = arg
-            friction_properties = [kwargs[key] for key in keys]
-        self.Subroutine('Friction', subroutine_type, subroutine_filepath, n_state_dependent_variables,
-                        friction_properties)
+    @typing.overload
+    def ShearGeneralStaticStep(self, time_period: float = 1.0, description: str = '',
+                               incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
+                               maximal_increments: int = 100, initial_increment_size: float = 1.0,
+                               minimal_increment_size: float = 1e-5, maximal_increment_size: float = 1.0):
+        pass
 
-    @staticmethod
-    def _setParameters(text: str, **kwargs):
-        for key, value in kwargs.items():
-            pattern = '\n{}\s?=\s?.+\n'.format(key)
-            if type(value) != str:
-                text = re.sub(pattern, '\n{} = {}\n'.format(key, value), text)
-            else:
-                text = re.sub(pattern, "\n{} = '{}'\n".format(key, value), text)
-        return text
+    @typing.overload
+    def ShearGeneralStaticStep(self, time_period: float = 1.0, description: str = '',
+                               incrementation_type=AbaqusBaseStepIncrementationType.Fixed,
+                               maximal_increments: int = 100, fixed_increment_size: float = 0.1):
+        pass
+
+    def ShearGeneralStaticStep(self, time_period: float = 1.0, description: str = '',
+                               incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
+                               *args, **kwargs):
+        self._GeneralStaticStep('Shear', time_period, description, incrementation_type, *args, **kwargs)
+
+    @typing.overload
+    def InitialStressExplicitDynamicStep(self, time_period: float = 1.0, description: str = '',
+                                         linear_bulk_viscosity: float = 0.06, quad_bulk_viscosity: float = 1.2,
+                                         scaling_factor: float = 1.0,
+                                         incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
+                                         maximal_time_increment: typing.Union[float, str] = 'UNLIMITED'):
+        pass
+
+    @typing.overload
+    def InitialStressExplicitDynamicStep(self, time_period: float = 1.0, description: str = '',
+                                         linear_bulk_viscosity: float = 0.06, quad_bulk_viscosity: float = 1.2,
+                                         scaling_factor: float = 1.0,
+                                         incrementation_type=AbaqusBaseStepIncrementationType.Fixed,
+                                         increment_size: typing.Union[
+                                             float, str] = 'ELEMENT_BY_ELEMENT_INCREMENTATION'):
+        pass
+
+    def InitialStressExplicitDynamicStep(self, time_period: float = 1.0, description: str = '',
+                                         linear_bulk_viscosity: float = 0.06, quad_bulk_viscosity: float = 1.2,
+                                         scaling_factor: float = 1.0,
+                                         incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
+                                         *args, **kwargs):
+        self._ExplicitDynamicStep('Initial-Stress', time_period, description, linear_bulk_viscosity,
+                                  quad_bulk_viscosity,
+                                  scaling_factor, incrementation_type, *args, **kwargs)
+
+    @typing.overload
+    def ShearExplicitDynamicStep(self, time_period: float = 1.0, description: str = '',
+                                 linear_bulk_viscosity: float = 0.06, quad_bulk_viscosity: float = 1.2,
+                                 scaling_factor: float = 1.0,
+                                 incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
+                                 maximal_time_increment: typing.Union[float, str] = 'UNLIMITED'):
+        pass
+
+    @typing.overload
+    def ShearExplicitDynamicStep(self, time_period: float = 1.0, description: str = '',
+                                 linear_bulk_viscosity: float = 0.06, quad_bulk_viscosity: float = 1.2,
+                                 scaling_factor: float = 1.0,
+                                 incrementation_type=AbaqusBaseStepIncrementationType.Fixed,
+                                 increment_size: typing.Union[
+                                     float, str] = 'ELEMENT_BY_ELEMENT_INCREMENTATION'):
+        pass
+
+    def ShearExplicitDynamicStep(self, time_period: float = 1.0, description: str = '',
+                                 linear_bulk_viscosity: float = 0.06, quad_bulk_viscosity: float = 1.2,
+                                 scaling_factor: float = 1.0,
+                                 incrementation_type=AbaqusBaseStepIncrementationType.Automatic,
+                                 *args, **kwargs):
+        self._ExplicitDynamicStep('Shear', time_period, description, linear_bulk_viscosity, quad_bulk_viscosity,
+                                  scaling_factor, incrementation_type, *args, **kwargs)
 
     def generateAbaqusPythonScript(self):
         """Generate abaqus-python script file to the work directory
@@ -581,9 +375,10 @@ class AbaqusDirectShearTest(AbaqusModelBase):
         with open(template_path, 'r+') as file:
             template_text = file.read()
 
+        subroutine_filename = os.path.split(self.subroutines['Friction'].subroutine_filepath)[-1]
         template_text = self._setParameters(
             template_text,
-            CALCULATION_METHOD=self.calculationMethodString(),
+            CALCULATION_METHOD=self._calculationMethodString(),
             SOIL_LENGTH=self.geometries['Soil'].length,  # SOIL GEOMETRY
             SOIL_WIDTH=self.geometries['Soil'].width,
             SOIL_HEIGHT=self.geometries['Soil'].height,
@@ -596,21 +391,58 @@ class AbaqusDirectShearTest(AbaqusModelBase):
             SOLID_ELASTIC_MODULUS=self.materials['Solid'].elastic.modulus,  # SOLID MATERIAL PROPERTIES
             SOLID_POISSON_RATIO=self.materials['Solid'].elastic.poisson_ratio,
             SOLID_DENSITY=self.materials['Solid'].density,
-            SOIL_MESH_METHOD=self.meshes['Soil'].methodString(),  # MESH PARAMETERS
-            SOIL_SEEDS_NUMBER=self.meshes['Soil'].seeds_number,
-            SOIL_SEEDS_SIZE=self.meshes['Soil'].seeds_size,
-            SOLID_MESH_METHOD=self.meshes['Solid'].methodString(),
-            SOLID_SEEDS_NUMBER=self.meshes['Solid'].seeds_number,
-            SOLID_SEEDS_SIZE=self.meshes['Solid'].seeds_size,
+            SOIL_MESH_METHOD=self.meshes['Soil'].general.methodString(),  # MESH PARAMETERS
+            SOIL_SEEDS_NUMBER=self.meshes['Soil'].general.seeds_number,
+            SOIL_SEEDS_SIZE=self.meshes['Soil'].general.seeds_size,
+            SOLID_MESH_METHOD=self.meshes['Solid'].general.methodString(),
+            SOLID_SEEDS_NUMBER=self.meshes['Solid'].general.seeds_number,
+            SOLID_SEEDS_SIZE=self.meshes['Solid'].general.seeds_size,
             VERTICAL_PRESSURE=self.loads['Vertical-Pressure'].amplitude,  # VERTICAL PRESSURE
-            PREDEFINED_STRESS=self.loads['Predefined-Confining-Stress'].predefined_stress,  # PREDEFINED STRESS
+            PREDEFINED_STRESS=self.loads['Predefined-Stress'].predefined_stress,  # PREDEFINED STRESS
             SUBROUTINE=self.subroutines['Friction'].subroutineString(),  # FRIC/VFRIC SUBROUTINE
-            SUBROUTINE_PATH=self.subroutines['Friction'].subroutine_filepath,
-            N_STATE_DEPENDENT_VARS=self.subroutines['Friction'].n_state_dependent_variables,
-            FRICTION_PARAMETERS=self.subroutines['Friction'].properties,
+            SUBROUTINE_PATH=subroutine_filename,
+            # N_STATE_DEPENDENT_VARS=self.subroutines['Friction'].n_state_dependent_variables,
+            # FRICTION_PARAMETERS=self.subroutines['Friction'].properties,
             FIELD_OUTPUT_VARIABLES=self.outputs['F-Output-1'].variables,
-            HISTORY_OUTPUT_VARIABLES=self.outputs['H-Output-1'].variables, 
-            EXPLICIT_NUM_INTERVALS=self.outputs['H-Output-1'].num_intervals,  # FIXME
+            HISTORY_OUTPUT_VARIABLES=self.outputs['H-Output-1'].variables,
+            EXPLICIT_NUM_INTERVALS=self.outputs['H-Output-1'].num_intervals,
+        )
+        template_text = self._setParameters_string_not_sensitive(
+            template_text,
+            # NORMAL CONTACT
+            normal_contactStiffness=self.contacts['Contact'].normal_contact.contactStiffness,
+            normal_pressureOverclosure=self.contacts['Contact'].normal_contact.pressureOverclosure,
+            normal_allowSeparation=self.contacts['Contact'].normal_contact.allowSeparation,
+            normal_maxStiffness=self.contacts['Contact'].normal_contact.maxStiffness,
+            normal_table=self.contacts['Contact'].normal_contact.table,
+            normal_constraintEnforcementMethod=self.contacts['Contact'].normal_contact.constraintEnforcementMethod,
+            normal_overclosureFactor=self.contacts['Contact'].normal_contact.overclosureFactor,
+            normal_overclosureMeasure=self.contacts['Contact'].normal_contact.overclosureMeasure,
+            normal_contactStiffnessScaleFactor=self.contacts['Contact'].normal_contact.contactStiffnessScaleFactor,
+            normal_initialStiffnessScaleFactor=self.contacts['Contact'].normal_contact.initialStiffnessScaleFactor,
+            normal_clearanceAtZeroContactPressure=self.contacts[
+                'Contact'].normal_contact.clearanceAtZeroContactPressure,
+            normal_stiffnessBehavior=self.contacts['Contact'].normal_contact.stiffnessBehavior,
+            normal_stiffnessRatio=self.contacts['Contact'].normal_contact.stiffnessRatio,
+            normal_upperQuadraticFactor=self.contacts['Contact'].normal_contact.upperQuadraticFactor,
+            normal_lowerQuadraticRatio=self.contacts['Contact'].normal_contact.lowerQuadraticRatio,
+            # TANGENTIAL CONTACT
+            tangential_formulation=self.contacts['Contact'].tangential_contact.formulation,
+            tangential_directionality=self.contacts['Contact'].tangential_contact.directionality,
+            tangential_slipRateDependency=self.contacts['Contact'].tangential_contact.slipRateDependency,
+            tangential_pressureDependency=self.contacts['Contact'].tangential_contact.pressureDependency,
+            tangential_temperatureDependency=self.contacts['Contact'].tangential_contact.temperatureDependency,
+            tangential_dependencies=self.contacts['Contact'].tangential_contact.dependencies,
+            tangential_exponentialDecayDefinition=self.contacts[
+                'Contact'].tangential_contact.exponentialDecayDefinition,
+            tangential_table=self.contacts['Contact'].tangential_contact.table,
+            tangential_shearStressLimit=self.contacts['Contact'].tangential_contact.shearStressLimit,
+            tangential_maximumElasticSlip=self.contacts['Contact'].tangential_contact.maximumElasticSlip,
+            tangential_fraction=self.contacts['Contact'].tangential_contact.fraction,
+            tangential_absoluteDistance=self.contacts['Contact'].tangential_contact.absoluteDistance,
+            tangential_elasticSlipStiffness=self.contacts['Contact'].tangential_contact.elasticSlipStiffness,
+            tangential_nStateDependentVars=self.contacts['Contact'].tangential_contact.nStateDependentVars,
+            tangential_useProperties=self.contacts['Contact'].tangential_contact.useProperties,
         )
         if self.calculation_method == AbaqusCalculationMethod.Standard:
             template_text = self._setParameters(
@@ -619,9 +451,12 @@ class AbaqusDirectShearTest(AbaqusModelBase):
                 INITIAL_STRESS_DESCRIPTION=self.steps['Initial-Stress'].description,
                 INITIAL_STRESS_INCREMENTATION_TYPE=self.steps['Initial-Stress'].incrementation.incrementationType(),
                 INITIAL_STRESS_MAXIMAL_INCREMENTS=self.steps['Initial-Stress'].incrementation.maximal_increments,
-                INITIAL_STRESS_INITIAL_INCREMENT_SIZE=self.steps['Initial-Stress'].incrementation.initial_increment_size,
-                INITIAL_STRESS_MINIMAL_INCREMENT_SIZE=self.steps['Initial-Stress'].incrementation.minimal_increment_size,
-                INITIAL_STRESS_MAXIMAL_INCREMENT_SIZE=self.steps['Initial-Stress'].incrementation.maximal_increment_size,
+                INITIAL_STRESS_INITIAL_INCREMENT_SIZE=self.steps[
+                    'Initial-Stress'].incrementation.initial_increment_size,
+                INITIAL_STRESS_MINIMAL_INCREMENT_SIZE=self.steps[
+                    'Initial-Stress'].incrementation.minimal_increment_size,
+                INITIAL_STRESS_MAXIMAL_INCREMENT_SIZE=self.steps[
+                    'Initial-Stress'].incrementation.maximal_increment_size,
                 INITIAL_STRESS_FIXED_INCREMENT_SIZE=self.steps['Initial-Stress'].incrementation.fixed_increment_size,
                 SHEAR_TIME_PERIOD=self.steps['Shear'].time_period,
                 SHEAR_DESCRIPTION=self.steps['Shear'].description,
@@ -637,7 +472,8 @@ class AbaqusDirectShearTest(AbaqusModelBase):
                 template_text, RP_VELOCITY=self.loads['Velocity'].velocity,
                 INITIAL_STRESS_TIME_PERIOD=self.steps['Initial-Stress'].time_period,
                 INITIAL_STRESS_DESCRIPTION=self.steps['Initial-Stress'].description,
-                INITIAL_STRESS_MAXIMAL_TIME_INCREMENT=self.steps['Initial-Stress'].incrementation.maximal_time_increment,
+                INITIAL_STRESS_MAXIMAL_TIME_INCREMENT=self.steps[
+                    'Initial-Stress'].incrementation.maximal_time_increment,
                 INITIAL_STRESS_INCREMENT_SIZE=self.steps['Initial-Stress'].incrementation.increment_size,
                 INITIAL_STRESS_SCALING_FACTOR=self.steps['Initial-Stress'].incrementation.scaling_factor,
                 INITIAL_STRESS_LINEAR_BULK_VISCOSITY=self.steps['Initial-Stress'].linear_bulk_viscosity,
@@ -670,7 +506,8 @@ class AbaqusDirectShearTest(AbaqusModelBase):
 
         output_template_text = output_template_text.replace('new directory', self.work_directory)
         pattern = '\n{}.+\n'.format('WORK_DIRECTORY\s?=\s?')
-        output_template_text = re.sub(pattern, '\n{}\n'.format("WORK_DIRECTORY = r'new directory'"), output_template_text)
+        output_template_text = re.sub(pattern, '\n{}\n'.format("WORK_DIRECTORY = r'new directory'"),
+                                      output_template_text)
         output_template_text = output_template_text.replace('new directory', self.work_directory)
         output_template_text = output_template_text.replace('dst.cae', self.model_name + '.cae')
 
@@ -683,33 +520,14 @@ class AbaqusDirectShearTest(AbaqusModelBase):
             file.write(output_template_text)
 
         try:
-            shutil.copy(os.path.join(self.old_work_directory, self.subroutines['Friction'].subroutine_filepath),
+            shutil.copy(os.path.join(self._old_work_directory, self.subroutines['Friction'].subroutine_filepath),
                         self.work_directory)
+
         except FileNotFoundError:
             pass
         except shutil.SameFileError:
             pass
 
-    def submit(self):
-        """Submit the abaqus job, 3 steps will be done in this function
-
-        - use abaqus command to run the model script file the build the abaqus model
-        - use abaqus command to submit the job
-        - use abaqus command to ru the output script file to obtain the output data and save it to the work directory
-
-        Returns
-        -------
-
-        """
-        os.system('abaqus cae noGUI={}.py'.format(self.model_name))
-        os.system('abaqus job=Job-1 user={} cpus=1 int double'.format(self.subroutines['Friction'].subroutine_filepath))
-        os.system('abaqus cae noGUI={}.py'.format(self.output_name))
-
-        for filename in os.listdir():
-            if os.path.splitext(filename)[-1] in ['.com', '.dat', '.log', '.msg', '.prt', '.rpy', '.sim', '.sta', '.1', 
-                                                  '.abq', '.mdl', '.pac', '.res', '.sel', '.stt']:
-                os.remove(filename)
-
     def plot(self):
         """Plot some figures when everything has been done properly, more specifically, the output data have been saved
         to the DATA folder in the work directory
@@ -719,8 +537,10 @@ class AbaqusDirectShearTest(AbaqusModelBase):
 
         """
         os.chdir(self.work_directory)
+        if self.contacts['Contact'].tangential_contact_type != AbaqusTangentialContactType.UserDefined:
+            return
 
-        friction_parameters = (parameter[0] for parameter in self.subroutines['Friction'].properties)
+        friction_parameters = tuple(self.contacts['Contact'].tangential_contact.properties)
         (cohesion, G0, R, e_ref0, Lambda, xi, phi_u, dd, n_p, n_d, e0, thickness) = friction_parameters
 
         if self.calculation_method == AbaqusCalculationMethod.Standard:
