@@ -80,6 +80,40 @@ RP_VELOCITY = [0.006, 0.0, 0.0, 0.0, 0.0, 0.0]
 VERTICAL_PRESSURE = 310.0
 PREDEFINED_STRESS = [-310.0, -310.0, -310.0, 0.0, 0.0, 0.0]
 
+# TODO CONTACT
+# NORMAL CONTACT
+normal_contactStiffness = DEFAULT
+normal_pressureOverclosure = HARD
+normal_allowSeparation = ON
+normal_maxStiffness = None
+normal_table = ()
+normal_constraintEnforcementMethod = DEFAULT
+normal_overclosureFactor = 0.0
+normal_overclosureMeasure = 0.0
+normal_contactStiffnessScaleFactor = 1.0
+normal_initialStiffnessScaleFactor = 1.0
+normal_clearanceAtZeroContactPressure = 0.0
+normal_stiffnessBehavior = LINEAR
+normal_stiffnessRatio = 0.01
+normal_upperQuadraticFactor = 0.03
+normal_lowerQuadraticRatio = 0.3333
+# TANGENTIAL CONTACT
+tangential_formulation = USER_DEFINED
+tangential_directionality = ISOTROPIC
+tangential_slipRateDependency = OFF
+tangential_pressureDependency = OFF
+tangential_temperatureDependency = OFF
+tangential_dependencies = 0
+tangential_exponentialDecayDefinition = COEFFICIENTS
+tangential_table = ((0.0,), (250.0,), (5.19,), (0.89,), (0.147,), (0.424,), (31.2,), (7.57,), (2.06,), (0.46,), (0.5771,), (0.00115,))
+tangential_shearStressLimit = None
+tangential_maximumElasticSlip = FRACTION
+tangential_fraction = 0.0
+tangential_absoluteDistance = 0.0
+tangential_elasticSlipStiffness = None
+tangential_nStateDependentVars = 28
+tangential_useProperties = ON
+
 # TODO USER-DEFINED SUBROUTINE
 SUBROUTINE = 'FRIC'
 SUBROUTINE_PATH = 'FRIC_exponential_CV.for'
@@ -89,8 +123,8 @@ FRICTION_PARAMETERS = ((0.0,), (250.0,), (5.19,), (0.89,), (0.147,), (0.424,), (
 # TODO OUTPUT
 STANDARD_FREQUENCY = 1
 EXPLICIT_NUM_INTERVALS = 50  # FIXME NOT DEFINED
-FIELD_OUTPUT_VARIABLES = ['S', 'E', 'LE', 'U', 'RF', 'RT', 'RM', 'P', 'CSTRESS', 'CDISP', 'CFORCE', 'CNAREA', 'CSTATUS'] # STANDARD
-# FIELD_OUTPUT_VARIABLES = ['S', 'E', 'LE', 'U', 'V', 'A', 'RF', 'P', 'CSTRESS', 'CFORCE', 'FSLIPR', 'FSLIP', 'PPRESS'] # EXPLICIT
+FIELD_OUTPUT_VARIABLES = ['S', 'E', 'LE', 'U', 'RF', 'RT', 'RM', 'P', 'CSTRESS', 'CDISP', 'CFORCE', 'CNAREA', 'CSTATUS']
+# FIELD_OUTPUT_VARIABLES = ['S', 'E', 'LE', 'U', 'V', 'A', 'RF', 'P', 'CSTRESS', 'CFORCE', 'FSLIPR', 'FSLIP', 'PPRESS']
 HISTORY_OUTPUT_VARIABLES = ['U1', 'RF1']
 
 # TODO WORK DIRECTORY
@@ -260,11 +294,95 @@ else:
 # TODO CONTACT
 # Contact property
 mdb.models['Model-1'].ContactProperty('IntProp-1')
-mdb.models['Model-1'].interactionProperties['IntProp-1'].NormalBehavior(pressureOverclosure=HARD, allowSeparation=ON,
+"""mdb.models['Model-1'].interactionProperties['IntProp-1'].NormalBehavior(pressureOverclosure=HARD, allowSeparation=ON,
                                                                         constraintEnforcementMethod=DEFAULT)
 mdb.models['Model-1'].interactionProperties['IntProp-1'].TangentialBehavior(
     formulation=USER_DEFINED, nStateDependentVars=N_STATE_DEPENDENT_VARS,
-    useProperties=ON, table=FRICTION_PARAMETERS)
+    useProperties=ON, table=FRICTION_PARAMETERS)"""
+
+if normal_pressureOverclosure == HARD:
+    mdb.models['Model-1'].interactionProperties['IntProp-1'].NormalBehavior(
+        pressureOverclosure=HARD,
+        allowSeparation=normal_allowSeparation,
+        constraintEnforcementMethod=normal_constraintEnforcementMethod)
+
+elif normal_pressureOverclosure == EXPONENTIAL:
+    mdb.models['Model-1'].interactionProperties['IntProp-1'].NormalBehavior(
+        pressureOverclosure=EXPONENTIAL,
+        table=normal_table,
+        maxStiffness=normal_maxStiffness,
+        constraintEnforcementMethod=normal_constraintEnforcementMethod)
+
+elif normal_pressureOverclosure == LINEAR:
+    mdb.models['Model-1'].interactionProperties['IntProp-1'].NormalBehavior(
+        pressureOverclosure=LINEAR,
+        contactStiffness=normal_contactStiffness,
+        constraintEnforcementMethod=normal_constraintEnforcementMethod)
+
+elif normal_pressureOverclosure == TABULAR:
+    mdb.models['Model-1'].interactionProperties['IntProp-1'].NormalBehavior(
+        pressureOverclosure=TABULAR,
+        table=normal_table,
+        constraintEnforcementMethod=normal_constraintEnforcementMethod)
+
+else:  # SCALE_FACTOR
+    mdb.models['Model-1'].interactionProperties['IntProp-1'].NormalBehavior(
+        pressureOverclosure=SCALE_FACTOR,
+        contactStiffnessScaleFactor=normal_contactStiffnessScaleFactor,
+        initialStiffnessScaleFactor=normal_initialStiffnessScaleFactor,
+        overclosureFactor=normal_overclosureFactor,
+        overclosureMeasure=normal_overclosureMeasure,
+        constraintEnforcementMethod=normal_constraintEnforcementMethod)
+
+if tangential_formulation == FRICTIONLESS:
+    mdb.models['Model-1'].interactionProperties['IntProp-1'].TangentialBehavior(
+        formulation=FRICTIONLESS)
+
+elif tangential_formulation == PENALTY:
+    mdb.models['Model-1'].interactionProperties['IntProp-1'].TangentialBehavior(
+        formulation=PENALTY,
+        directionality=tangential_directionality,
+        slipRateDependency=tangential_slipRateDependency,
+        pressureDependency=tangential_pressureDependency,
+        temperatureDependency=tangential_temperatureDependency,
+        dependencies=tangential_dependencies,
+        table=tangential_table,
+        shearStressLimit=tangential_shearStressLimit,
+        maximumElasticSlip=tangential_maximumElasticSlip,
+        fraction=tangential_fraction,
+        absoluteDistance=tangential_absoluteDistance,
+        elasticSlipStiffness=tangential_elasticSlipStiffness)
+
+elif tangential_formulation == EXPONENTIAL_DECAY:
+    mdb.models['Model-1'].interactionProperties['IntProp-1'].TangentialBehavior(
+        formulation=EXPONENTIAL_DECAY,
+        exponentialDecayDefinition=tangential_exponentialDecayDefinition,
+        table=tangential_table,
+        maximumElasticSlip=tangential_maximumElasticSlip,
+        fraction=tangential_fraction,
+        absoluteDistance=tangential_absoluteDistance,
+        elasticSlipStiffness=tangential_elasticSlipStiffness)
+
+elif tangential_formulation == ROUGH:
+    mdb.models['Model-1'].interactionProperties['IntProp-1'].TangentialBehavior(
+        formulation=ROUGH)
+
+elif tangential_formulation == LAGRANGE:
+    mdb.models['Model-1'].interactionProperties['IntProp-1'].TangentialBehavior(
+        formulation=LAGRANGE, directionality=tangential_directionality,
+        slipRateDependency=tangential_slipRateDependency,
+        pressureDependency=tangential_pressureDependency,
+        temperatureDependency=tangential_temperatureDependency,
+        dependencies=tangential_dependencies,
+        table=tangential_table,
+        shearStressLimit=tangential_shearStressLimit)
+else:  # USER_DEFINED
+    mdb.models['Model-1'].interactionProperties['IntProp-1'].TangentialBehavior(
+        formulation=USER_DEFINED,
+        nStateDependentVars=tangential_nStateDependentVars,
+        useProperties=tangential_useProperties,
+        table=tangential_table)
+
 
 # Contact assignment
 assembly = mdb.models['Model-1'].rootAssembly
